@@ -16,13 +16,13 @@
  * @author Akinobu LEE
  * @date   Thu Feb 10 19:36:47 2005
  *
- * $Revision: 1.2 $
+ * $Revision: 1.9 $
  * 
  */
 /*
- * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2011 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2011 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -68,7 +68,7 @@
 /// @ingroup hmminfo
 
 /// Possible maximum value of state ID (in unsigned short)
-#define MAX_STATE_NUM 65535
+#define MAX_STATE_NUM 2147483647
 
 /// Delimiter strings/characters to generate logical triphone names
 #define HMM_RC_DLIM "+"		///< Right context delimiter in string
@@ -80,7 +80,7 @@
 #define SPMODEL_NAME_DEFAULT "sp"
 
 /// Length limit of HMM name (including ones generated in Julius)
-#define MAX_HMMNAME_LEN 128
+#define MAX_HMMNAME_LEN 256
 
 /// Specify method of calculating approximated acoustic score at inter-word context pseudo phones on word edge
 enum iwcd_type {
@@ -95,7 +95,7 @@ enum iwcd_type {
 /// Stream information (although current Julius supports only single stream)
 typedef struct {
   short num;			///< Number of stream
-  short vsize[50];		///< Vector size for each stream
+  short vsize[MAXSTREAMNUM];	///< Vector size for each stream
 } HTK_HMM_StreamInfo;
 
 /// %HMM Option
@@ -112,6 +112,7 @@ typedef struct _HTK_HMM_trans {
   char *name;			///< Name (NULL if not defined as Macro)
   short statenum;		///< Number of state
   PROB **a;			///< Matrix of transition probabilities
+  int id; 			///< Uniq transition id starting from 0
   struct _HTK_HMM_trans *next;  ///< Pointer to next data, NULL if last
 } HTK_HMM_Trans;
 
@@ -137,19 +138,42 @@ typedef struct _HTK_HMM_dens {
   struct _HTK_HMM_dens *next;	///< Pointer to next data, NULL if last
 } HTK_HMM_Dens;
 
+/// %HMM stream weight definition
+typedef struct _HTK_HMM_stream_weight {
+  char *name;			///< Name (NULL for in-line definition)
+  VECT *weight;			///< Weight of each stream in log scale
+  short len;			///< Length of above
+  struct _HTK_HMM_stream_weight *next; ///< Pointer to next data, NULL on last
+} HTK_HMM_StreamWeight;
+
 /**
- * @brief %HMM state data
- *
+ * @brief %HMM mixture PDF for a stream
+ * 
  * @note
  * In a tied-mixture model, @a b points to a codebook defined as GCODEBOOK
  * intead of the array of densities.
+ * 
+ */
+typedef struct _HTK_HMM_PDF {
+  char *name;			///< Name (NULL for in-line definition)
+  boolean tmix;			///< TRUE if this is assigned to tied-mixture codebook
+  short stream_id;		///< Stream ID to which this pdf is assigned, begins from 0
+  short mix_num;		///< Number of densities (mixtures) assigned.
+  HTK_HMM_Dens **b;		///< Link array to assigned densities, or pointer to GCODEBOOK in tied-mixture model
+  PROB *bweight;		///< Weights corresponding to above
+  struct _HTK_HMM_PDF *next;	///< Pointer to next data, or NULL at last
+} HTK_HMM_PDF;
+
+/**
+ * @brief %HMM state data
+ *
  */
 typedef struct _HTK_HMM_state {
   char *name;			///< Name (NULL if not defined as Macro)
-  short mix_num;		///< Number of densities (mixtures) assigned
-  HTK_HMM_Dens **b;		///< Link array to assigned densities, or pointer to GCODEBOOK in tied-mixture model
-  PROB *bweight;		///< Weights corresponding to above
-  unsigned short id;		///< Uniq state id starting from 0 for caching of output probability
+  short nstream;		///< Num of stream
+  HTK_HMM_StreamWeight *w;	///< Pointer to stream weight data, or NULL is not specified
+  HTK_HMM_PDF **pdf;	        ///< Array of mixture PDFs for each stream
+  int id; 			///< Uniq state id starting from 0 for caching of output probability
   struct _HTK_HMM_state *next;  ///< Pointer to next data, NULL if last
 } HTK_HMM_State;
 
@@ -243,6 +267,7 @@ typedef struct _cd_set{
 } CD_Set;
 /// Top structure to hold all the %HMM sets
 typedef struct {
+  boolean binary_malloc;	///< TRUE if read from binary
   APATNODE *cdtree;		///< Root of index tree for name lookup
 } HMM_CDSET_INFO;
 //@}
@@ -316,6 +341,8 @@ typedef struct {
   HTK_HMM_Trans *trstart;	///< Root pointer to the list of transition matrixes
   HTK_HMM_Var *vrstart;		///< Root pointer to the list of variance data
   HTK_HMM_Dens *dnstart;	///< Root pointer to the list of density (mixture) data
+  HTK_HMM_PDF *pdfstart;	///< Root pointer to the list of mixture pdf data
+  HTK_HMM_StreamWeight *swstart; ///< Root pointer to the list of stream weight data
   HTK_HMM_State *ststart;	///< Root pointer to the list of state data
   HTK_HMM_Data *start;		///< Root pointer to the list of models
   //@}
@@ -333,7 +360,9 @@ typedef struct {
   //@{
   APATNODE *tr_root;		///< Root index node for transition matrixes
   APATNODE *vr_root;		///< Root index node for variance data
+  APATNODE *sw_root;		///< Root index node for stream weight data
   APATNODE *dn_root;		///< Root index node for density data
+  APATNODE *pdf_root;		///< Root index node for mixture PDF
   APATNODE *st_root;		///< Root index node for state data
   APATNODE *physical_root;	///< Root index node for defined %HMM name
   APATNODE *logical_root;	///< Root index node for logical %HMM name
@@ -362,11 +391,13 @@ typedef struct {
   LOGPROB iwsp_penalty;		///< Extra ransition penalty for interword skippable short pause insertion for multi-path mode
   boolean variance_inversed;	///< TRUE if variances are inversed
   
+  int totaltransnum;		///< Total number of transitions
   int totalmixnum;		///< Total number of defined mixtures
   int totalstatenum;		///< Total number of states
   int totalhmmnum;		///< Total number of physical %HMM
   int totallogicalnum;		///< Total number of logical %HMM
   int totalpseudonum;		///< Total number of pseudo %HMM
+  int totalpdfnum;		///< Total number of mixture PDF
   int codebooknum;		///< Total number of codebook on tied-mixture model
   int maxcodebooksize;		///< Maximum size of codebook on tied-mixture model
   int maxmixturenum;		///< Maximum number of Gaussian per mixture
@@ -374,10 +405,24 @@ typedef struct {
 
   BMALLOC_BASE *mroot;		///< Pointer for block memory allocation
   BMALLOC_BASE *lroot;		///< Pointer for block memory allocation for logical HMM
+  BMALLOC_BASE *cdset_root;		///< Pointer for block memory allocation for logical HMM
+
+  int *tmp_mixnum;		///< Work area for state reading
+
+#ifdef ENABLE_MSD
+  boolean has_msd;		///< TRUE if this model contains MSD part
+#endif
+
+  void *hook;			///< General purpose hook
 
   //@}
 } HTK_HMM_INFO;
 
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* init_phmm.c */
 void htk_hmm_set_pause_model(HTK_HMM_INFO *hmminfo, char *spmodel_name);
 /* rdhmmdef.c */
@@ -385,43 +430,59 @@ void rderr(char *str);
 char *read_token(FILE *fp);
 boolean rdhmmdef(FILE *, HTK_HMM_INFO *);
 void htk_hmm_inverse_variances(HTK_HMM_INFO *hmm);
+#ifdef ENABLE_MSD
+void htk_hmm_check_msd(HTK_HMM_INFO *hmm);
+#endif
 /* rdhmmdef_options.c */
-void set_global_opt(FILE *fp, HTK_HMM_INFO *hmm);
+boolean set_global_opt(FILE *fp, HTK_HMM_INFO *hmm);
 char *get_cov_str(short covtype);
 char *get_dur_str(short durtype);
 /* rdhmmdef_trans.c */
-void trans_add(HTK_HMM_INFO *hmm, HTK_HMM_Trans *new);
+void trans_add(HTK_HMM_INFO *hmm, HTK_HMM_Trans *newParam);
 HTK_HMM_Trans *get_trans_data(FILE *, HTK_HMM_INFO *);
 void def_trans_macro(char *, FILE *, HTK_HMM_INFO *);
 /* rdhmmdef_state.c */
 HTK_HMM_State *get_state_data(FILE *, HTK_HMM_INFO *);
 void def_state_macro(char *, FILE *, HTK_HMM_INFO *);
 HTK_HMM_State *state_lookup(HTK_HMM_INFO *hmm, char *keyname);
-void state_add(HTK_HMM_INFO *hmm, HTK_HMM_State *new);
+void state_add(HTK_HMM_INFO *hmm, HTK_HMM_State *newParam);
+/* rdhmmdef_mpdf.c */
+void mpdf_add(HTK_HMM_INFO *hmm, HTK_HMM_PDF *newParam);
+HTK_HMM_PDF *mpdf_lookup(HTK_HMM_INFO *hmm, char *keyname);
+HTK_HMM_PDF *get_mpdf_data(FILE *fp, HTK_HMM_INFO *hmm, int mix_num, short stream_id);
+void def_mpdf_macro(char *name, FILE *fp, HTK_HMM_INFO *hmm);
 /* rdhmmdef_dens.c */
 HTK_HMM_Dens *get_dens_data(FILE *, HTK_HMM_INFO *);
 void def_dens_macro(char *, FILE *, HTK_HMM_INFO *);
 HTK_HMM_Dens *dens_lookup(HTK_HMM_INFO *hmm, char *keyname);
-void dens_add(HTK_HMM_INFO *hmm, HTK_HMM_Dens *new);
+void dens_add(HTK_HMM_INFO *hmm, HTK_HMM_Dens *newParam);
 /* rdhmmdef_var.c */
 HTK_HMM_Var *get_var_data(FILE *, HTK_HMM_INFO *);
 void def_var_macro(char *, FILE *, HTK_HMM_INFO *);
-void var_add(HTK_HMM_INFO *hmm, HTK_HMM_Var *new);
+void var_add(HTK_HMM_INFO *hmm, HTK_HMM_Var *newParam);
+/* rdhmmdef_streamweight.c */
+HTK_HMM_StreamWeight *get_streamweight_data(FILE *fp, HTK_HMM_INFO *hmm);
+void def_streamweight_macro(char *, FILE *, HTK_HMM_INFO *);
+void sw_add(HTK_HMM_INFO *hmm, HTK_HMM_StreamWeight *newParam);
 /* rdhmmdef_data.c */
 void def_HMM(char *, FILE *, HTK_HMM_INFO *);
 HTK_HMM_Data *htk_hmmdata_new(HTK_HMM_INFO *);
-void htk_hmmdata_add(HTK_HMM_INFO *hmm, HTK_HMM_Data *new);
+void htk_hmmdata_add(HTK_HMM_INFO *hmm, HTK_HMM_Data *newParam);
 /* rdhmmdef_tiedmix.c */
-void tmix_read(FILE *fp, HTK_HMM_State *state, HTK_HMM_INFO *hmm);
-void codebook_add(HTK_HMM_INFO *hmm, GCODEBOOK *new);
+void tmix_read(FILE *fp, HTK_HMM_PDF *mpdf, HTK_HMM_INFO *hmm);
+void codebook_add(HTK_HMM_INFO *hmm, GCODEBOOK *newParam);
 /* rdhmmdef_regtree.c */
 void def_regtree_macro(char *name, FILE *fp, HTK_HMM_INFO *hmm);
 /* rdhmmdef_hmmlist.c */
 boolean rdhmmlist(FILE *fp, HTK_HMM_INFO *hmminfo);
+boolean save_hmmlist_bin(FILE *fp, HTK_HMM_INFO *hmminfo);
+boolean load_hmmlist_bin(FILE *fp, HTK_HMM_INFO *hmminfo);
+
 /* put_htkdata_info.c */
 void put_htk_trans(FILE *fp, HTK_HMM_Trans *t);
 void put_htk_var(FILE *fp, HTK_HMM_Var *v);
 void put_htk_dens(FILE *fp, HTK_HMM_Dens *d);
+void put_htk_mpdf(FILE *fp, HTK_HMM_PDF *m);
 void put_htk_state(FILE *fp, HTK_HMM_State *s);
 void put_htk_hmm(FILE *fp, HTK_HMM_Data *h);
 void put_logical_hmm(FILE *fp, HMM_Logical *l);
@@ -455,10 +516,10 @@ char *leftcenter_name(char *hmmname, char *buf);
 char *rightcenter_name(char *hmmname, char *buf);
 
 /* CD_SET related */
-boolean regist_cdset(APATNODE **root, HTK_HMM_Data *d, char *cdname);
+boolean regist_cdset(APATNODE **root, HTK_HMM_Data *d, char *cdname, BMALLOC_BASE **mroot);
 boolean make_cdset(HTK_HMM_INFO *hmminfo);
 void put_all_cdinfo(HTK_HMM_INFO *hmminfo);
-void free_cdset(APATNODE **root);
+void free_cdset(APATNODE **root, BMALLOC_BASE **mroot);
 CD_Set *cdset_lookup(HTK_HMM_INFO *hmminfo, char *cdstr);
 CD_Set *lcdset_lookup_by_hmmname(HTK_HMM_INFO *hmminfo, char *hmmname);
 CD_Set *rcdset_lookup_by_hmmname(HTK_HMM_INFO *hmminfo, char *hmmname);
@@ -475,4 +536,9 @@ int param_check_and_adjust(HTK_HMM_INFO *hmm, HTK_Param *pinfo, boolean vflag);
 boolean write_binhmm(FILE *fp, HTK_HMM_INFO *hmm, Value *para);
 boolean read_binhmm(FILE *fp, HTK_HMM_INFO *hmm, boolean gzfile_p, Value *para);
 
+#ifdef __cplusplus
+}
+#endif
+
+
 #endif /* __SENT_HTK_HMM_2_H__ */
